import { types, template } from '@babel/core';
import { addNamed } from '@babel/helper-module-imports';
import { isRequired } from '@babel/helper-compilation-targets';

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;
  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }
  return target;
}

const _excluded = ["node"];
function isEmpty(obj) {
  return Object.keys(obj).length === 0;
}
const isRequiredOptions = {
  compatData: {
    webIMR: {
      chrome: "105.0.0",
      edge: "105.0.0",
      firefox: "106.0.0",
      opera: "91.0.0",
      safari: "16.4.0",
      opera_mobile: "72.0.0",
      ios: "16.4.0",
      samsung: "20.0",
      deno: "1.24.0"
    },
    nodeIMR: {
      node: "20.6.0"
    },
    nodeFSP: {
      node: "10.0.0"
    }
  }
};
const SUPPORT_CACHE = new WeakMap();
function getSupport(targets) {
  if (SUPPORT_CACHE.has(targets)) return SUPPORT_CACHE.get(targets);
  const {
      node: nodeTarget
    } = targets,
    webTargets = _objectWithoutPropertiesLoose(targets, _excluded);
  const emptyNodeTarget = nodeTarget == null;
  const emptyWebTargets = isEmpty(webTargets);
  const needsNodeSupport = !emptyNodeTarget || emptyWebTargets;
  const needsWebSupport = !emptyWebTargets || emptyNodeTarget;
  const webSupportsIMR = !emptyWebTargets && !isRequired("webIMR", webTargets, isRequiredOptions);
  const nodeSupportsIMR = !emptyNodeTarget && !isRequired("nodeIMR", {
    node: nodeTarget
  }, isRequiredOptions);
  const nodeSupportsFsPromises = !emptyNodeTarget && !isRequired("nodeFSP", {
    node: nodeTarget
  }, isRequiredOptions);
  const result = {
    needsNodeSupport,
    needsWebSupport,
    nodeSupportsIMR,
    webSupportsIMR,
    nodeSupportsFsPromises
  };
  SUPPORT_CACHE.set(targets, result);
  return result;
}

function imp(path, name, module) {
  return addNamed(path, name, module, {
    importedType: "es6"
  });
}
const imr = s => template.expression.ast`
  import.meta.resolve(${s})
`;
const imrWithFallback = s => template.expression.ast`
  import.meta.resolve?.(${s}) ?? new URL(${types.cloneNode(s)}, import.meta.url)
`;
function importToPlatformApi(targets, transformers, toCommonJS) {
  const {
    needsNodeSupport,
    needsWebSupport,
    nodeSupportsIMR,
    webSupportsIMR,
    nodeSupportsFsPromises
  } = getSupport(targets);
  let buildFetchAsync;
  let buildFetchSync;
  const p = ({
    web: w,
    node: n,
    nodeFSP: nF = nodeSupportsFsPromises,
    webIMR: wI = webSupportsIMR,
    nodeIMR: nI = nodeSupportsIMR,
    toCJS: c = toCommonJS
  }) => +w + (+n << 1) + (+wI << 2) + (+nI << 3) + (+c << 4) + (+nF << 5);
  const readFileP = (fs, arg) => {
    if (nodeSupportsFsPromises) {
      return template.expression.ast`${fs}.promises.readFile(${arg})`;
    }
    return template.expression.ast`
      new Promise(
        (a =>
          (r, j) => ${fs}.readFile(a, (e, d) => e ? j(e) : r(d))
        )(${arg})
      )`;
  };
  switch (p({
    web: needsWebSupport,
    node: needsNodeSupport,
    webIMR: webSupportsIMR,
    nodeIMR: nodeSupportsIMR,
    toCJS: toCommonJS
  })) {
    case p({
      web: true,
      node: true
    }):
      buildFetchAsync = specifier => {
        const web = transformers.webFetch(types.callExpression(types.identifier("fetch"), [(webSupportsIMR ? imr : imrWithFallback)(types.cloneNode(specifier))]));
        const node = nodeSupportsIMR ? template.expression.ast`
                import("fs").then(
                  fs => ${readFileP(types.identifier("fs"), template.expression.ast`new URL(${imr(specifier)})`)}
                ).then(${transformers.nodeFsAsync()})
              ` : template.expression.ast`
                Promise.all([import("fs"), import("module")])
                  .then(([fs, module]) =>
                    ${readFileP(types.identifier("fs"), template.expression.ast`
                        module.createRequire(import.meta.url).resolve(${specifier})
                      `)}
                  )
                  .then(${transformers.nodeFsAsync()})
              `;
        return template.expression.ast`
            typeof process === "object" && process.versions?.node
              ? ${node}
              : ${web}
          `;
      };
      break;
    case p({
      web: true,
      node: false,
      webIMR: true
    }):
      buildFetchAsync = specifier => transformers.webFetch(types.callExpression(types.identifier("fetch"), [imr(specifier)]));
      break;
    case p({
      web: true,
      node: false,
      webIMR: false
    }):
      buildFetchAsync = specifier => transformers.webFetch(types.callExpression(types.identifier("fetch"), [imrWithFallback(specifier)]));
      break;
    case p({
      web: false,
      node: true,
      toCJS: true
    }):
      buildFetchSync = specifier => transformers.nodeFsSync(template.expression.ast`
            require("fs").readFileSync(require.resolve(${specifier}))
          `);
      buildFetchAsync = specifier => template.expression.ast`
            require("fs").promises.readFile(require.resolve(${specifier}))
              .then(${transformers.nodeFsAsync()})
          `;
      break;
    case p({
      web: false,
      node: true,
      toCJS: false,
      nodeIMR: true
    }):
      buildFetchSync = (specifier, path) => transformers.nodeFsSync(template.expression.ast`
            ${imp(path, "readFileSync", "fs")}(
              new URL(${imr(specifier)})
            )
          `);
      buildFetchAsync = (specifier, path) => template.expression.ast`
          ${imp(path, "promises", "fs")}
            .readFile(new URL(${imr(specifier)}))
            .then(${transformers.nodeFsAsync()})
        `;
      break;
    case p({
      web: false,
      node: true,
      toCJS: false,
      nodeIMR: false
    }):
      buildFetchSync = (specifier, path) => transformers.nodeFsSync(template.expression.ast`
            ${imp(path, "readFileSync", "fs")}(
              ${imp(path, "createRequire", "module")}(import.meta.url)
                .resolve(${specifier})
            )
          `);
      buildFetchAsync = (specifier, path) => transformers.webFetch(template.expression.ast`
            ${imp(path, "promises", "fs")}
              .readFile(
                ${imp(path, "createRequire", "module")}(import.meta.url)
                  .resolve(${specifier})
              )
          `);
      break;
    default:
      throw new Error("Internal Babel error: unreachable code.");
  }
  buildFetchAsync ??= buildFetchSync;
  const buildFetchAsyncWrapped = (expression, path) => {
    if (types.isStringLiteral(expression)) {
      return template.expression.ast`
        Promise.resolve().then(() => ${buildFetchAsync(expression, path)})
      `;
    } else {
      return template.expression.ast`
        Promise.resolve(\`\${${expression}}\`).then((s) => ${buildFetchAsync(types.identifier("s"), path)})
      `;
    }
  };
  return {
    buildFetch: buildFetchSync || buildFetchAsync,
    buildFetchAsync: buildFetchAsyncWrapped,
    needsAwait: !buildFetchSync
  };
}
function buildParallelStaticImports(data, needsAwait) {
  if (data.length === 0) return null;
  const declarators = [];
  if (data.length === 1) {
    let rhs = data[0].fetch;
    if (needsAwait) rhs = types.awaitExpression(rhs);
    declarators.push(types.variableDeclarator(data[0].id, rhs));
  } else if (needsAwait) {
    const ids = data.map(({
      id
    }) => id);
    const fetches = data.map(({
      fetch
    }) => fetch);
    declarators.push(types.variableDeclarator(types.arrayPattern(ids), types.awaitExpression(template.expression.ast`
            Promise.all(${types.arrayExpression(fetches)})
          `)));
  } else {
    for (const {
      id,
      fetch
    } of data) {
      declarators.push(types.variableDeclarator(id, fetch));
    }
  }
  return types.variableDeclaration("const", declarators);
}

export { buildParallelStaticImports, importToPlatformApi };
//# sourceMappingURL=index.js.map
